/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Structures and global variables ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

static GList *my_pixmaps_directories = NULL;

GtkWidget *main_window;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'gui_statistics.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* write a dog or kennel statistics page to dog_info.
   the sorting has already been done previously.
   type = 10: show kennel statistics.
   type = 20: take into account all dogs
   type 0 - 5: take into account only dogs on corresponding levels.
   there are two pages of statistics. for dogs: one with name, kennel,
   fastest times in season, points, wins and number of races; the other
   one with name, kennel, fastest times ever, points/race, wins/race
   and number of races.
   for kennels: the first page just like for dogs; the second one
   with average weighted time points/race, wins/race and number of
   races */
gint write_statistics(GtkWidget *widget, gint type,
		     gint attribute);

/* sort all dogs by an attribute.
   attribute = 20: fastest sprgint in current season
   attribute = 30: fastest 2-lap race in current season
   attribute = 40: fastest 3-lap race in current season
   attribute = 50: points
   attribute = 60: wins
   attribute = 70: fastest sprgint ever
   attribute = 80: fastest 2-lap race ever
   attribute = 90: fastest 3-lap race ever
   attribute = 100: points / race
   attribute = 110: wins / race.
   time values we multiply by -1
   as we want them to be sorted ascending */
gint sort_dogs_statistics(gint attribute, gint *sort_order_all);
 
/* fill in a row with dog statistics */
gint append_dog_to_statistics_list(GtkCList *clist, dog Dog, gint page);

gint append_kennel_to_statistics_list(GtkCList *clist,
				     gint kennel_id, gint attribute);

/* if type is not 10, write the first 20 dogs according to
   'sort_order', 'type' (i.e. all dogs or those on a given level)
   and attribute to dog_info.
   if type is 10, write kennel statistics according to page
   to dog_info */
gint write_statistics_list(GtkWidget *widget, gint type,
			  gint *sort_order, gint attribute);

/* find out which page to show when the user browses the statistics
   and show it. if attribute = 0 and left_or_right is -1 or 1, the
   user has turned the page; if attribute is > 0, he has clicked on
   a column to sort the dogs.
   if both are 0, he just went to the statistics screen from somewhere.
   type = 10: show kennel statistics
   type = 20 or 0-5: show dog statistics */
gint show_statistics(GtkWidget *widget, gint attribute,
		    gint left_or_right, gint type);

/* display the current standings in the champions' race
   (i.e., the first 6 kennels in the categories best WAT,
   most points / race and most wins / race) */
gint show_champions_race(GtkWidget *widget);

/* write the best 6 kennels in the categories 'WAT',
   p/r and w/r to dog_info and mark the qualified kennels
   with color */
gint write_champions_race(GtkWidget *widget, gint best_kennels[][6]);

/* show the records in 'dog_info' */
gint show_all_time_records(GtkWidget *widget);

/* called the semi-month before the trophy races.
   allocate memory for a new history element and fill it
   except for the fastest dog times, for those could be
   topped in the trophy races that come */
gint write_new_history(gint dog_history);



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'startup.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* start a new game: create dogs and kennels, set default options etc.
   if new_game is 1, we start a new game from within a game */
gint start(GtkWidget *widget, gint new_game);



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Functions from 'gui_functions.c' ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* Use this function to set the directory containing installed pixmaps. */
void
my_add_pixmap_directory                   (const gchar     *directory);

/* This is an internally used function to find pixmap files. */
gchar*
my_find_pixmap_file                       (const gchar     *filename);

/* enter informations about a dog into a 'clist' */
gint append_dog_to_dog_list(dog Dog, GtkCList *dog_list);

/* fill in a list with the dogs of a given kennel
   in a given order; either the dog_list window (dog_info = 0)
   or the dog_info window (dog_info = 1);
   type = 0, 1, 2: show all dogs 
   type = 3: show only males (when interbreeding)
   type = 4: show only females in heat (when interbreeding) */
gint write_dog_list(GtkWidget *widget, gint kennel_id, gint dog_info,
		   gint type);

/* write a message into the message window */
gint print_message(GtkWidget *widget, gchar *message);

/* get an integer from a label */
gint get_int_from_label(GtkLabel *label);

/* prgint detailed information about a dog into the
   dog_info list in the main window;
   called when the human player selects a dog in the
   dog_list list */
gint print_dog_info(dog Dog, GtkCList *dog_info);

/* get order index of the currently selected dog */
gint get_selected_row(GtkWidget *widget, gint dog_info);

/* write id of the currently selected dog */
gint write_selected_row(GtkWidget *widget,
		       gint new_selected_row, gint dog_info);

/* change the title of the dog_info list.
   it's can be either:
   type 0: the title of a detailed dog information window
   type 10-29: a kennel
   type 30: race results shown at the end of a semi-month
   type 31: a race list of entered dogs
   type 32: results of last races */
gint write_dog_info_label(GtkWidget *widget, gint level_or_kennel_id,
			 gint laps, gint race_number, gint type);

/* write new kennel id and owner in the dog list label */
gint write_dog_list_label(GtkWidget *widget, gint new_kennel_id);

/* write a list of either a race result or of a race yet to come */
gint write_race_list(GtkCList *dog_info, gint race_type,
		    gint race_number, gint type);

/* 1st column: name 
   2nd: best time this season or race time
   3rd: dog's kennel
   4th: either remarks like 'best time ever', 'level up' etc., or,
   if it's a trophy race, points of each kennel */
gint append_dog_to_race_list(GtkCList *dog_info, gint race_type,
			    gint race_number, gint index, gint type);

/* used in 'append_dog_to_race_list' */
gint write_race_time(gint race_type, gint race_number, gint idx, gchar **dest);

/* write race remarks to 'dest' and note special events
   like level up etc. in '*color';
   used in 'append_dog_to_race_list' */
gint write_race_remarks(gint race_type, gint race_number,
		       gint idx, gint type, gint *color, gchar **dest);


/* format 'dog_info' fittingly to display
   type = 0: detailed dog info
   type = 10 - 29: a cpu-kennel dog list
   type = 30: race results
   type = 31: race entries
   type = 32: last races */
gint format_dog_info(GtkWidget *widget, gint *width, gchar title[][16],
		    gint level_or_kennel_id, gint laps, gint race_number,
		    gint type);

/* return the order index corresponding to the selected row */
gint get_order_index_from_row(GtkCList *clist, gint kennel_id, gint row);

/* show either race results or entry lists in the dog_info window */
gint display_races(GtkWidget *widget, gint race_number, gint type);

/* if the user's entering dogs in races,
   we write additional information in shorter form
   into the message window */
gint print_short_dog_info(GtkWidget *widget, gint race_number, dog Dog);

/* things that happen between race weekends */
gint between_race_weekends(GtkWidget *widget);

/* set the colors of clist rows containing dog information
   (dogs on dog_list or dog_info) */
gint change_colors_clist(GtkCList *clist, gint dog_info);

/* change colors of a row */
gint change_color_row(GtkCList *clist, gint row, gint type);

/* provide the fitting color for 'change_color_row' and
   other functions */
gint set_color(GdkColor *color, gint type);

/* mark first row of a race list with color depending
   on the number of laps so that the user doesn't have to
   decipher the race_laps_number label */
gint set_laps_style(GtkCList *dog_info, gint number_of_laps);

/* write a dog list of a kennel into a window.
   if doginfo is 0, the user is simply browsing
   through the kennels in dog_list.
   otherwise he's browsing through the cpu-kennels
   for trading or breeding.
   type = 0 just browsing kennels, looking at other's dogs
   type = 1 trading
   type = 2 breeding */ 
gint display_kennel(GtkWidget *widget, gint new_kennel_id,
		   gint doginfo, gint type);

/* remove selected dog from the user's kennel */
gint remove_dog(GtkWidget *widget);

gint change_widget_color(GtkWidget *widget, gint type);

/* return TRUE if selected dogs can (technichally) be mated,
   FALSE otherwise */
gboolean set_sensitive_mate_dogs(GtkWidget *widget);

/* find out the indices of the selected dogs on the
   two lists dog_list and dog_info */
gint get_dog_indices(GtkWidget *widget, gint *indices);

/* prgint some news in a pop-up window */
gint print_news(GtkWidget *widget, gchar *news_text, gint type);

/* set all buttons insensitive, so that other functions
   only have to look after setting some to sensitive */
gint set_buttons_non_sensitive(GtkWidget *widget);

/* set the sensitivity of buttons depending on the menu the user's
   in and the selected rows on the two lists */
gint set_buttons(GtkWidget *widget);

/* return TRUE if the dog may participate in the race (fitting level,
   status etc.), FALSE else */
gboolean set_sensitive_enter_dogs(gint dog_index, gint race_number);

/* justify the (first) eight columns of 'clist' according to the
 integer values.
 col = 0: justify left
 col = 1: centered
 col = 2: right */
gint justify_clist(GtkCList *clist, gint col_1, gint col_2, gint col_3,gint col_4,
		  gint col_5,gint col_6,gint col_7,gint col_8);

/* show various popups with news for the user at the
   beginning of a new race-weekend */
gint news(GtkWidget *widget);

/* show a small window with a progress bar advancing every 1/3 of a second
   for 'duration' seconds. 'text' is written above the progress bar */
gint show_progress(gint duration, gchar *text);

/* show an ok_or_cancel window with text 'text' and type 'type' */
gint show_ok_or_cancel_window(GtkWidget *widget, gchar *text, gint type);

/* if the season is over except the trophy races and
   the human player's first in one of the kennel categories,
   note it in the two 'selected_row' labels */
gint awards_news(GtkWidget *widget);

/* nullify statistics and points etc. */
gint season_ending(GtkWidget *widget);

/* determine which season history to show and go ahead */
gint show_history(GtkWidget *widget, gint season);

/* show a page of history (from a history_element) in dog_info */
gint show_season_history(GtkWidget *widget, history_element *element,
			gint seasonnr);

/* show a popup with the trophy winner */
gint popup_trophy_winner(GtkWidget *widget);

/* type = 0: set the buttons in the options window corresponding to
             the values of the global option variables when the options
	     window is popped up
   type = 10: set the values corresponding to the buttons when the user
              clicks 'OK' in the options window */
gint set_buttons_options_window(GtkWidget *widget, gint type);

/* return the last row of a clist that
   contains something */
gint clist_last_row(GtkCList *clist);

/* choose the next race upwards from current_race
   in which there's a dog from the user's kennel */
gint next_user_race(gint current_race, gint type, gint direction);

/* return the next user race including 'current_race' (contrary to the
   above function) the level of which is switched
   on for live races */
gint next_user_race_levels(gint current_race);

/* find the next kennel with a bitch in heat upwards or
   downwards from current_kennel */
gint next_kennel_in_heat(gint current_kennel, gint direction);

/* pop up a window with help text in it */
gint display_help(GtkWidget *widget, gint to_show);

/* write the main window labels after loading a game */
gint write_labels(GtkWidget *widget);

/* set the label that indicates whether the current game is saved
   or not to 1 (saved) or 0 (not).
   if 'status' is -1 we simply return the current status */
gint set_save_status(GtkWidget *widget, gint status);

/* change the 'popups_active' label */
gint change_popups_active(GtkWidget *widget, gint difference);

/* show a 'live race': a couple of progress bars racing against each
   other */
gint show_live_race(GtkWidget *live_race_window, gint race_number);

/* generate a growing sequence of fractions from 0 to 1 which
   give the race progress of a dog; the smaller max_idx the faster
   the race time */
gint generate_update_fractions(gfloat *update_fractions, gint max_idx);

/* set some race data and the dog names in the live race window */
gint set_live_race_labels(GtkWidget *live_race_window,
			  gint race_nr);

/* write pointers to the respective widgets in the live race window
   into the arguments */
gint get_live_race_pointers(GtkWidget *live_race_window,
			    GtkWidget **progress_bars, GtkWidget **dog_names,
			    GtkWidget **dog_labels, GtkWidget **buttons);

/* set or append 'number' as text into 'label' */
gint label_set_text_from_int(GtkLabel *label, gint number, gint append);

/* set or append 'text' into 'label' */
gint label_set_text(GtkLabel *label, gchar *text, gint append);

/* change the color of a progress bar */
gint set_color_progress_bar(GtkWidget *progress_bar, gint type);

